<?php
namespace Controller;

use Util\Utilities;
use Util\Messages;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\Log\Exception;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;

class Resource extends \Generated\Controller\Sponsor {
    /** @var \Service\Resource $resourceService*/
	protected $resourceService;
	protected $resourceProjectService;
	protected $profileService;
	protected $localeService;
	protected $environment;

	public function __construct(\Service\Resource $resourceService, \Service\ResourceProject $resourceProjectService,
                                 \Service\Profile $profileService,\Service\Locale $localeService, \Service\Structure $structureService,
								\Util\Utilities $environment, \Zeedhi\Framework\DataSource\Manager $dataSourceManager) {
		$this->resourceService = $resourceService;
    	$this->resourceProjectService = $resourceProjectService;
    	$this->profileService = $profileService;
    	$this->localeService = $localeService;
		$this->structureService = $structureService;
		$this->environment = $environment;
		parent::__construct($dataSourceManager);
	}

	public function findResourceToScreen(Request\Filter $request, Response $response){
		$dataSet = array();
		try{
			$filter = $request->getFilterCriteria()->getConditions();
			$dataSet = $this->resourceService->getResources($filter);
		} catch(\Exception $ex){
			$response->addMessage(new Response\Message("Não foi possível achar seus recursos, pois " . $ex->getMessage()));
		}
		$response->addDataSet(new DataSet('resources', $dataSet));
	}

	public function findResourceAssetType(Request\Filter $request, Response $response){
		try{
			$dataSet = $this->resourceService->getResourceAssetTypes();
		} catch(\Exception $ex){
			$response->addMessage(new Response\Message("Não foi possível achar seus tipos de material, pois " . $ex->getMessage()));
		}
		$response->addDataSet(new DataSet('findResourceAssetType', $dataSet));
	}
	public function findAllEmployee(Request\Filter $request, Response $response){
		try{
            $filter = $request->getFilterCriteria()->getConditions();
			$dataSet = $this->resourceService->findAllEmployee($filter);
            $response->addDataSet(new DataSet('findResourceAssetType', $dataSet));
		} catch(\Exception $ex){
			$response->addMessage(new Response\Message("Não foi possível achar seus tipos de material, pois " . $ex->getMessage()));
		}
	}

	public function getResourceData(Request\Filter $request, Response $response){
		$filterData = $request->getFilterCriteria()->getConditions();
		$resourceId = $filterData[0]['value'];
		try{
			$dataSet = $this->resourceService->getFullResourceDataFromId($resourceId);
			$response->addDataSet(new DataSet('ResourceInfo', $dataSet));
		} catch(\Exception $ex){
			$response->addMessage(new Response\Message("Não foi possível achar seus dados para visualização, pois " . $ex->getMessage()));
		}
	}

	public function findAccessProfile(Request\Filter $request, Response $response){
		$filterData = $request->getFilterCriteria()->getConditions();
		try{
			$dataSet = $this->resourceService->findAccessProfile($filterData);
            DevOpsUtilities::setDataSetDefault($request,$response,$dataSet);
		} catch(\Exception $ex){
			$response->setError(new Response\Error ("Não foi possível achar seus dados para visualização, pois " . $ex->getMessage(),$ex->getCode()));
		}
	}

	public function findAccessProfileOfResource(Request\Filter $request, Response $response){
		$filterData = $request->getFilterCriteria()->getConditions();
		try{
            $resourceId = DevOpsUtilities::getValueInFilterDataSourceByName($filterData,'RESOURCE',true);
			$dataSet = $this->resourceService->findAccessProfileOfResource($resourceId,$filterData);
            DevOpsUtilities::setDataSetDefault($request,$response,$dataSet);
		} catch(\Exception $ex){
			$response->setError(new Response\Error ("Não foi possível achar seus dados para visualização, pois " . $ex->getMessage(),$ex->getCode()));
		}
	}

    public function removeAccessProfileOfResource(Request\DataSet $request, Response $response ){
        try{
            $row = $request->getDataSet()->getRows()[0];
            $id = $row['ID'];
            $this->resourceService->removeAccessProfileOfResource($id);

        } catch(\Exception $ex){
            $response->setError(new Response\Error ($ex->getMessage(),$ex->getCode()));
        }

    }

    public function saveAccessProfileOfResource(Request\DataSet $request, Response $response)
    {
        try {
            $row = $request->getDataSet()->getRows()[0];
            $nrperfilacesso = $row['NRPERFILACESSO'];
            $resourceId = $row['RESOURCE_ID'];
            $this->resourceService->saveAccessProfileOfResource($resourceId, $nrperfilacesso);
        } catch (\Exception $ex) {
            $response->setError(new Response\Error ($ex->getMessage(), $ex->getCode()));
        }
    }

    public function allResourcesOrganization(Request\Filter $request, Response $response){
		try{
            $filterData = $request->getFilterCriteria()->getConditions();
            $project = Utilities::getValueInFilterDataSourceByName($filterData, 'PROJECT', true);
            $dataSet = $this->resourceService->allResourcesOrganization($filterData);
            $response->addDataSet(new DataSet('ResourceInfo', $dataSet));
        } catch(\Exception $ex){
            $response->addMessage(new Response\Message("Não foi possível achar seus dados para visualização, pois " . $ex->getMessage()));
        }
	}

    public function ResourcesOrganization(Request\Filter $request, Response $response){
        $filterData = $request->getFilterCriteria()->getConditions();
        $resourceId = $filterData[0]['value'];
        try{
			$dataSet = $this->resourceService->ResourcesOrganization($resourceId);
		} catch(\Exception $ex){
			$response->addMessage(new Response\Message("Não foi possível achar seus dados para visualização, pois " . $ex->getMessage()));
		}
		$response->addDataSet(new DataSet('ResourceInfo', $dataSet));
	}


	public function saveResource(Request\DataSet $request, Response $response){
		$dataSet = $request->getDataSet()->getRows();
		$param = $dataSet[0];
		$password = (md5($param['PASSWORD']));
		$id = empty($param['ID']) ? null : $param['ID'];
		$externalId = empty($param['EXTERNAL_ID']) ? null : $param['EXTERNAL_ID'];
        $rating = empty($param['RATING']) ? null : $param['RATING'];
		$email = empty($param['EMAIL']) ? null : $param['EMAIL'];
		$structure = empty($param['STRUCTURE']) ? null : $param['STRUCTURE'];
		$permission = empty($param['PERMISSION']) ? null : $param['PERMISSION'];
		$assetType = empty($param['ASSET_TYPE']) ? null : $param['ASSET_TYPE'];
		$description = empty($param['DESCRIPTION']) ? null : $param['DESCRIPTION'];
        $profiles = empty($param['PROFILE_NAME']) ? [] : $param['PROFILE_NAME'];
        $availableFrom = empty($param['AVAILABLE_FROM']) ? null :  \DateTime::createFromFormat(Utilities::FORMAT_DATE_DEFAULT, $param['AVAILABLE_FROM'] . ' 00:00:00');

        try{
            $resource = $this->resourceService->persistResource($id, $externalId, $param['TYPE_CONTROL'], $param['STATUS'], $email, $password, $param['NAME'], $permission,
											$description, $structure, $assetType, $profiles,$availableFrom, $rating);
            $response->addDataSet(new DataSet('ResourceInfo', array($resource->getId())));
			$response->addNotification(new Response\Notification(\Util\Messages::SUC_SUCCESSFUL_SAVED));
        } catch(\Exception $ex){
			$response->addMessage(new Response\Message($ex->getMessage(), Response\Message::TYPE_ERROR));
			throw $ex;
        }
	}

    public function findNotProfileUsers(Request\Filter $request, Response $response){
    	$filter = $request->getFilterCriteria()->getConditions();
    	$profile = empty($filter[0]) ? 0 : $filter[0]['value'];
		try{
			$dataSet = $this->resourceService->getNotProfileUsers($profile);
		} catch(\Exception $ex){
			$response->addMessage(new Response\Message("Não foi possível achar seus recursos, pois " . $ex->getMessage()));
		}
		$response->addDataSet(new DataSet('ResourceInfo', $dataSet));
	}

    public function fetchCompleteResourceInformation(Request\Filter $request, Response $response){
        $filterData = $request->getFilterCriteria()->getConditions();
        $resourceId = $filterData[0]['value'];
        try{
            $dataSet = $this->resourceService->getFullResourceDataFromId($resourceId);
        } catch(\Exception $ex){
            $response->addMessage(new Response\Message("Não foi possível achar seus dados para visualização, pois " . $ex->getMessage()));
        }
        $response->addDataSet(new DataSet('ResourceInfo', $dataSet));
        $resourceProjectPaths = $this->resourceProjectService->getAllResourceProjectsPaths($resourceId);
        $response->addDataSet(new DataSet('resourceProjectsPaths', $resourceProjectPaths));
    }

    public function deleteResource(Request\Row $request, Response $response) {
        $row = $request->getRow();
        $resourceId = $row['ID'];
        try {
            $this->resourceService->deleteResource($resourceId);
        } catch(\Exception $ex) {
            throw new Exception($ex->getMessage());
        }

        $response->addMessage(new Response\Message("Recurso deletado com sucesso!"));
    }

	public function resourcesSponsors(Request\Filter $request, Response $response){
		$dataSet = $this->resourceService->findResourceSponsorByOrganization();
		$response->addDataset(new DataSet('resourceSponsor', $dataSet));
	}

	public function deleteResourceAvailability(Request\DataSet $request, Response $response){
		$count = 0;
		$dataSet = $request->getDataSet()->getRows();
		foreach ($dataSet as $param) {
			try{
				$this->resourceService->removeResourceAvailability($param['ID']);
			}catch(\Exception $ex){
				$response->addMessage(new Response\Message(\Util\Messages::ERR_NOT_SUCCESSFUL_DELETED_EXPLANATION . $ex->getMessage()));
				$count--;
			}
			$count++;
		}
		$response->addMessage(new Response\Message("$count disponibilidades deletadas."));
	}

	public function deleteResourceCost(Request\DataSet $request, Response $response){
		$count = 0;
		$dataSet = $request->getDataSet()->getRows();
		foreach ($dataSet as $param) {
			try{
				$effectiveDate = $param['ALL_DAYS'] === 'A' ? null : $param['EFFECTIVE_DATE'];
				$this->resourceService->removeResourceCost($effectiveDate);
			}catch(\Exception $ex){
				$response->addMessage(new Response\Message(\Util\Messages::ERR_NOT_SUCCESSFUL_DELETED_EXPLANATION . $ex->getMessage()));
				$count--;
			}
			$count++;
		}
		$response->addMessage(new Response\Message("$count custos deletados."));
	}

	public function findResourcesAllocated(Request\Filter $request, Response $response){
		$dataSet = $request->getFilterCriteria()->getConditions();
		$task = $dataSet[0]['value'];
		$dataSet = $this->resourceService->getResourcesAllocatedInTask($task);
		$response->addDataset(new DataSet('resources', $dataSet));
	}

	public function getAllocatedResources(Request\Filter $request, Response $response){
		$conditions = $request->getFilterCriteria()->getConditions();
		$resourcesIDs = $conditions[0]['value'];
		try{
			$dataSet = $this->resourceService->returnOnlyAllocatedResources($resourcesIDs);
		} catch(\Exception $ex){
			$response->addMessage(new Response\Message("Não foi possível achar seus recursos, pois " . $ex->getMessage()));
		}
		$response->addDataSet(new DataSet('allocatedResources', $dataSet));
	}

	public function saveAvailability(Request\DataSet $request, Response $response){
		$dataSet = $request->getDataSet()->getRows();
		$param = $dataSet[0];
		$id = empty($param['ID']) ? null : $param['ID'];
		try{
			$this->resourceService->persistAvailability($id, $param['BEGIN'], $param['END'], $param['RESOURCE']);
		} catch(\Exception $ex){
			$response->addMessage(new Response\Message(\Util\Messages::ERR_NOT_SUCCESSFUL_SAVED_EXPLANATION . $ex->getMessage()));
		}
		$response->addMessage(new Response\Message(\Util\Messages::SUC_SUCCESSFUL_SAVED));
	}

	public function saveCost(Request\DataSet $request, Response $response){
		$dataSet = $request->getDataSet()->getRows();
		$param = $dataSet[0];
		try{
			$effectiveDate = $param['ALL_DAYS'] === 'A' ? null : $param['EFFECTIVE_DATE'];
			$this->resourceService->persistCostResource($param['DEFAULT_TAX'], $param['HOUR_COST'], $param['EXTRA_HOUR_TAX'], $effectiveDate,
														$param['ALL_DAYS'], $param['RESOURCE']);
		} catch(\Exception $ex){
			$response->addMessage(new Response\Message(\Util\Messages::ERR_NOT_SUCCESSFUL_SAVED_EXPLANATION . $ex->getMessage()));
		}
		$response->addMessage(new Response\Message(\Util\Messages::SUC_SUCCESSFUL_SAVED));
	}
	public function getResourceAvailabilities(Request\Filter $request, Response $response){
		$filterData = $request->getFilterCriteria()->getConditions();

		$resourceId = $filterData[0]['value'];

		try{
			$dataSet = $this->resourceService->getResourceAvailableFromId($resourceId);
		} catch(\Exception $ex){
			$response->addMessage(new Response\Message("Não foi possível achar seus dados para visualização, pois " . $ex->getMessage()));
		}

		$response->addDataSet(new DataSet('Available', $dataSet));
	}
	public function getResourceCosts(Request\Filter $request, Response $response){
		$filterData = $request->getFilterCriteria()->getConditions();

		$resourceId = $filterData[0]['value'];

		try{
			$dataSet = $this->resourceService->getResourceCostFromId($resourceId);
		} catch(\Exception $ex){
			$response->addMessage(new Response\Message("Não foi possível achar seus dados para visualização, pois " . $ex->getMessage()));
		}

		$response->addDataSet(new DataSet('Cost', $dataSet));
	}
	public function getResourceExceptions(Request\Filter $request, Response $response){
		$filterData = $request->getFilterCriteria()->getConditions();

		$resourceId = $filterData[0]['value'];

		try{
			$dataSet = $this->resourceService->getResourceExceptionFromId($resourceId);
		} catch(\Exception $ex){
			$response->addMessage(new Response\Message("Não foi possível achar seus dados para visualização, pois " . $ex->getMessage()));
		}

		$response->addDataSet(new DataSet('Exception', $dataSet));
	}
	public function getResourceCalendar(Request\Filter $request, Response $response){
		$filterData = $request->getFilterCriteria()->getConditions();

		$resourceId = $filterData[0]['value'];

		try{
			$dataSet = $this->resourceService->getResourceCalendarFromId($resourceId);
		} catch(\Exception $ex){
			$response->addMessage(new Response\Message("Não foi possível achar seus dados para visualização, pois " . $ex->getMessage()));
		}

		$response->addDataSet(new DataSet('Calendar', $dataSet));
	}
	public function getResourceProfiles(Request\Filter $request, Response $response){
		$filterData = $request->getFilterCriteria()->getConditions();

		$resourceId = $filterData[0]['value'];

		try{
			$dataSet = $this->resourceService->getResourceProfileFromId($resourceId);
		} catch(\Exception $ex){
			$response->addMessage(new Response\Message("Não foi possível achar seus dados para visualização, pois " . $ex->getMessage()));
		}

		$response->addDataSet(new DataSet('Profile', $dataSet));
	}

	public function saveResourceProfile(Request\DataSet $request, Response $response){
		$dataSet = $request->getDataSet()->getRows();
		$param = $dataSet[0];
		try{
			$this->resourceService->persistResourceProfile($param['RESOURCE'], $param['PROFILE']);
		} catch(\Exception $ex){
			$response->addMessage(new Response\Message(\Util\Messages::ERR_NOT_SUCCESSFUL_SAVED_EXPLANATION . $ex->getMessage()));
		}
		$response->addNotification(new Response\Notification(\Util\Messages::SUC_SUCCESSFUL_SAVED));
	}

	public function removeResourceProfile(Request\DataSet $request, Response $response){
		$count = 0;
		$dataSet = $request->getDataSet()->getRows();

		foreach ($dataSet as $param) {
			try{
				$this->resourceService->removeResourceProfile($param['ID']);
			}catch(\Exception $ex){
				$response->addMessage(new Response\Message(\Util\Messages::ERR_NOT_SUCCESSFUL_SAVED_EXPLANATION . $ex->getMessage()));
				$count--;
			}

			$count++;
		}
		$response->addMessage(new Response\Message("$count perfis deletados."));
	}

    public function findResourcesByOrganization(Request\Filter $request, Response $response) {
    	$resources = $this->resourceService->findResourcesByOrganization();
    	$response->addDataset(new DataSet('findResourcesByOrganization', $resources));
    }

    public function findAllStructureManagers(Request\Filter $request, Response $response) {
        $filters = $request->getFilterCriteria()->getConditions();
    	$resources = $this->resourceService->findAllStructureManagers($filters);
    	$response->addDataset(new DataSet('findResourcesByOrganization', $resources));
    }

    public function findUserData(Request\Filter $request, Response $response) {
        $userData = $this->resourceService->getUserData();
        $response->addDataSet(new DataSet('findUserData', $userData));
    }

    public function changePassword(Request\DataSet $request, Response $response) {
        $dataSet = $request->getDataSet()->getRows();
        $param = $dataSet[0];

        $newPassword = $param['NEW_PASSWORD'];
        $currentPassword = empty($dataSet[0]['PASSWORD']) ? null : $param['PASSWORD'];
        $resourceId = empty($dataSet[0]['RESOURCE_ID']) ? null : $param['RESOURCE_ID'];
        $result = empty($currentPassword)
			? $this->resourceService->resetResourcePassword($newPassword, $resourceId)
			: $this->resourceService->changePassword($currentPassword, $newPassword);

        if($result) {
            $response->addNotification(new Response\Notification("Senha alterada com sucesso"));
        } else {
			$response->addMessage(new Response\Message("A senha atual informada está incorreta."));
		}
    }

    public function saveEditedUserData(Request\DataSet $request, Response $response) {
        $dataSet = $request->getDataSet()->getRows();
        $param = $dataSet[0];
        $name = $param['NAME'];
        $email = $param['EMAIL'];
        $photo = empty($param['PHOTO_VIEW']) ? null : $param['PHOTO_VIEW'];
        try{
            $this->resourceService->saveEditedUserDataValidate($name, $email, $photo);
            $userData = $this->resourceService->getUserData();
            $response->addDataSet(new DataSet('userData', $userData));
            $response->addNotification(new Response\Notification("Perfil alterado com sucesso"));
        }catch (\Exception $e){
            $response->addMessage(new Response\Message($e->getMessage()));
        }

    }

	public function login(Request\Filter $request, Response $response) {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $email = $conditions[0]['value'];
            $password = $conditions[1]['value'];
            $resultSet = $this->resourceService->login($email, $password);
            if($resultSet){
                $response->addDataSet(new DataSet('login', array($resultSet)));
            } else {
                $response->addMessage(new Response\Message('Email ou senha inválido.'));
            }
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage()));
        }
    }

	public function refreshSession(Request $request, Response $response){
		$resultSet = $this->resourceService->refreshSession();
		if($resultSet){
			$response->addDataSet(new DataSet('login', array($resultSet)));
		} else {
			$response->addMessage(new Response\Message('Não foi possível fazer login.'));
		}
	}
	public function checkIsAlreadyLogged(Request\Filter $request, Response $response){
		if($this->resourceService->hasSession('userId') && $this->resourceService->hasSession('organizationId')){
			$resultSet = $this->resourceService->prepareOrganization($this->resourceService->getEnvironment()->getCurrentUser());
			$response->addDataSet(new DataSet('login', array($resultSet)));
		}
		else {
			$response->addDataSet(new DataSet('login', array()));
		}
	}
	public function logout(Request\Filter $request, Response $response){
		$this->clearSession();
	}
    public function clearSession(){
        $this->resourceService->destroySession();
    }
    public function sponsorCity( Request\Filter $request,Response $response){

        $param = $request->getFilterCriteria()->getConditions();
        $state = empty($param[0]) ? null : $param[0]['value'];
        if($state){
            $dataSet = $this->localeService->getCitiesByState($state);
        }else{
            $dataSet = array();
        }

        $response->addDataset(new DataSet('city', $dataSet));
    }
    public function sponsorState( Request\Filter $request,Response $response){

        $param = $request->getFilterCriteria()->getConditions();
        $countryId = empty($param[0]) ? null : $param[0]['value'];
        if($countryId){
            $dataSet =$this->localeService->getStatesByCountry($countryId);
        }else{
            $dataSet = array();
        }
        $response->addDataset(new DataSet('state', $dataSet));
    }
    public function sponsorCountry( Request\Filter $request,Response $response){

        $response->addDataset(new DataSet('country', $this->localeService->getAllCountries()));
    }

    public function findSponsor(Request\Filter $request, Response $response){
        $sponsors = $this->resourceService->getSponsors();
        $response->addDataSet(new DataSet('sponsor', $sponsors));
    }

    public function findSponsorProfile(Request\Filter $request, Response $response) {
        $sponsorProfiles = $this->profileService->getProfileByName(self::SPONSOR_PARTNER);
        $response->addDataSet(new DataSet('sponsorProfile', $sponsorProfiles));
    }

    public function getProfileByCurrentUser(Request\Filter $request, Response $response)
    {
        try {
            $projectStatus = $this->resourceService->getProfileByCurrentUser();
            $response->addDataSet(new DataSet('userProfile', $projectStatus));
        } catch (\Exception $error) {
            $response->addMessage(new Response\Message(\Util\Messages::ERR_NOT_SUCCESSFUL_FOUND_EXPLANATION . $error->getMessage()));
        }
    }

    public function saveSponsor(Request\DataSet $request,Response $response){
        try{
            $dataSet = $request->getDataSet()->getRows();
            $param = $dataSet[0];
            $login = empty($param['LOGIN'])? null:$param['LOGIN'];
            $password = empty($param['PASSWORD'])? null:$param['PASSWORD'] ;
            $status = empty($param['STATUS'])? 'A':$param['STATUS'] ;
            $structure = null;
            $name = empty($param['NAME'])? null:$param['NAME'];
            $nickName = empty($param['NICK_NAME'])? null:$param['NICK_NAME'];
            $documentNumber = empty($param['DOCUMENT_NUMBER'])? null:$param['DOCUMENT_NUMBER'];
            $description = empty($param['DESCRIPTION'])? null:$param['DESCRIPTION'];
            
            if(!empty($param['ID'])){
                $resourceId = $param['ID'];
                $resourceId = $this->resourceService->editSponsor($login, $password, $status, $name, $nickName, $documentNumber, $description, $resourceId);
                $message = "Alterações salvas com sucesso";
                $response->addDataSet(new DataSet('ResourceInfo', array($resourceId)));
            }else{
                $resource = $this->resourceService->newSponsor($login, $password, $status, $name, $nickName, $documentNumber, $description);
                $message = "Patrocinador criado!";
                $response->addDataSet(new DataSet('ResourceInfo', array($resource->getId())));
            }
            $response->addNotification(new Response\Notification($message));
        }catch (\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

	public function basicSponsor(Request\Filter $request,Response $response){
        $param = $request->getFilterCriteria()->getConditions();
        $resource = $param[0]["value"];
        $dataSetBasic = array();
        if($resource){
	        $dataSetBasic = $this->resourceService->basicSponsor($resource);
        }
        $response->addDataset(new DataSet('basicSponsor', $dataSetBasic));
    }

    public function findContacts(Request\Filter $request,Response $response){

        $param = $request->getFilterCriteria()->getConditions();
        $resource = Utilities::getValueInFilterDataSourceByName($param, "sponsor", true);
        $filters = DevOpsUtilities::createQueryFiltered($param);
        $dataSetContact = $this->resourceService->formContactList($resource, $filters);
        $response->addDataset(new DataSet('contactList', $dataSetContact));
    }

    public function editSponsorData( Request\Filter $request,Response $response)
    {

        $param = $request->getFilterCriteria()->getConditions();
        $sponsorId = $param[0]["value"];

        $dataSetBasic = $this->resourceService->basicSponsorEdit($sponsorId);
        $dataSetAdvanced = $this->resourceService->advancedSponsor($sponsorId);

        $response->addDataset(new DataSet('basicSponsor', $dataSetBasic));
        $response->addDataset(new DataSet('advancedSponsor', $dataSetAdvanced));
    }

    public function sponsorProjectData(Request\Filter $request,Response $response)
    {
        $filters = $request->getFilterCriteria()->getConditions();
        $sponsorId = DevOpsUtilities::getValueInFilterDataSourceByName($filters, 'SPONSOR', true);
        $conditions = DevOpsUtilities::createQueryFiltered($filters);
        $dataSetBasic = $this->resourceService->sponsorProjectData($sponsorId, $conditions);
        DevOpsUtilities::setDataSetDefault($request, $response, $dataSetBasic);
    }
    
    public function sponsorActive( Request\Filter $request,Response $response){

        $dataSetBasic = $this->resourceService->sponsorActive();

        $response->addDataset(new DataSet('basicSponsor', $dataSetBasic));
    }
    public function formContact( Request\Filter $request,Response $response){

        $param = $request->getFilterCriteria()->getConditions();
        $dataSetBasic = $this->resourceService->formContact($param[0]["value"]);

        $response->addDataset(new DataSet('formContact', $dataSetBasic));
    }

    public function saveContactSponsorList(Request\DataSet $request,Response $response)
    {
        try{
            
            $dataSet=$request->getDataSet()->getRows();
            $param = $dataSet[0];
    
            $sponsorId = empty($param['SPONSOR'])? null:$param['SPONSOR'];
            $address = empty($param['ADDRESS'])? null:$param['ADDRESS'];
            $city    = empty($param['CITY'])? null:$param['CITY'];
            $district= empty($param['DISTRICT'])? null:$param['DISTRICT'];
            $email   = empty($param['EMAIL'])? null:$param['EMAIL'];
            $name    = empty($param['NAME'])? null:$param['NAME'];
            $phone   = empty($param['PHONE'])? null:$param['PHONE'];
            $status  = empty($param['STATUS']) ? null : $param['STATUS'];
            $zipCode = empty($param['ZIP_CODE'])? null:$param['ZIP_CODE'];
            if(!empty($param['CONTACT'])){
                $contact = $param['CONTACT'];
                $contactId = $this->resourceService->updateContact($contact, $sponsorId, $address, $city, $district, $email, $name, $phone, $status, $zipCode);
            } else {
                $contactId = $this->resourceService->createNewContact($sponsorId, $address, $city, $district, $email, $name, $phone, $status, $zipCode);
            }
            $response->addDataset(new DataSet("contact", $contactId));
            $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_SAVED));
        } catch (\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteSponsor(Request\Row $request,Response $response){
        $row = $request->getRow();
        try{
            $this->resourceService->deleteSponsor($row['ID']);
            $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_REMOVED));
        }catch (\Exception $e){
            $response->addMessage(new Response\Message($e->getMessage()));
        }
    }

    public function deleteContact(Request\DataSet $request,Response $response){
        $dataSet = $request->getDataSet()->getRows();
        try{
            foreach($dataSet as $param){
                $sponsor = $this->resourceService->deleteSponsor($param['CONTACT']);
            }
            $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_REMOVED));
        }catch (\Exception $e){
            $response->addMessage(new Response\Message($e->getMessage()));
        }
    }

	public function findManager(Request\Filter $request, Response $response){
        $resourceExternalIds = $this->resourceService->findManagerExternalResourceIdsByIdentityProfiles();
        $dataSet = $this->resourceService->findManagerByOrganization($resourceExternalIds);
        $response->addDataset(new DataSet('managerProject', $dataSet));
    }

	public function findManagerProjectEdit(Request\Filter $request, Response $response){

        $dataSet = $this->resourceService->findManagerExternalResourceIdsByIdentityProfiles();
        $response->addDataset(new DataSet('managerProject', $dataSet));
    }
    public function findManagerProjectEditFilter(Request\Filter $request, Response $response){
        $filter = $request->getFilterCriteria()->getConditions();
        $dataSet = $this->resourceService->findResourceManagerProfilesByOrganization($filter);
        $response->addDataset(new DataSet('managerProjectFilter', $dataSet));

    }

	public function findResourcesForProject(Request\Filter $request, Response $response){
        $param = $request->getFilterCriteria()->getConditions();
        $projectId = $param[0]['value'];
        $resources = $this->resourceService->getDataSetResourcesForProject($projectId);
        $response->addDataSet(new DataSet('resourcesForProject', $resources));
    }

    public function projectResource(Request\Filter $request, Response $response){
        $filter = $request->getFilterCriteria()->getConditions();
        $projectId = $filter[0]['value'];
        $taskClassificationControl = Utilities::getValueInFilterDataSourceByName($filter, 'TASK_CLASSIFICATION', true);
        $filterNames = array();
        if($filter[count($filter) -1 ]['operator'] == 'LIKE_ALL') {
            $filterNames = explode("|",$filter[1]['columnName']);
            $filterOperator = "LIKE";
            $filterValue = $filter[count($filter) -1 ]['value'];
        } else if($filter[count($filter) -1 ]['operator'] == 'LIKE'){
            array_push($filterNames,$filter[1]['columnName']);
            $filterOperator = "LIKE";
            $filterValue = $filter[count($filter) -1 ]['value'];
        }else{
            $filterNames = false;
            $filterOperator = false;
            $filterValue = false;
        }

        $resources = $this->resourceService->getDataSetForEditProjectResource($projectId, $filterNames, $filterOperator, $filterValue, $taskClassificationControl);
        $response->addDataSet(new DataSet('projectResource', $resources));
    }

    public function findResourceProjects(Request\Filter $request, Response $response){
        $param = $request->getFilterCriteria()->getConditions();
        $user = $param[0]['value'];
        $resourceProjects = $this->resourceService->getAllResourceProjects($user);
        $response->addDataSet(new DataSet('resourceProjects', $resourceProjects));
    }

	public function findStructureByResourceId(Request\DataSet $request, Response $response){
		$dataSet = $request->getDataSet()->getRows();
		$resourceId = $dataSet[0]['responsibleId'];
		$structures = $this->resourceService->findStructureByResourceId($resourceId);
		$response->addDataSet(new DataSet('structures', $structures));
	}

	public function findSponsorWithCrud(Request\Filter $request, Response $response){
		$request->getFilterCriteria()->addCondition("ORGANIZATION", "=", $this->environment->getCurrentOrganizationId());
		$request->getFilterCriteria()->addCondition("CLASS", "=", "S");
		$request->getFilterCriteria()->addCondition("STATUS", "<>", "D");
		return parent::find($request, $response);
	}

	public function findSponsorWithCrudActive(Request\Filter $request, Response $response){
		$request->getFilterCriteria()->addCondition("ORGANIZATION", "=", $this->environment->getCurrentOrganizationId());
		$request->getFilterCriteria()->addCondition("CLASS", "=", "S");
		$request->getFilterCriteria()->addCondition("STATUS", "=", "A");
		return parent::findSetNameDataSet($request, $response,'findSponsorWithCrudActive');
	}

    public function removeSponsorWithCrud(Request\Row $request, Response $response){
         parent::delete($request, $response);
    }

    public function findFollowerWithCrud(Request\Filter $request, Response $response){
        $request->getFilterCriteria()->addCondition("ORGANIZATION", "=", $this->environment->getCurrentOrganization());
        $request->getFilterCriteria()->addCondition("STATUS", "<>", "D");
        return parent::find($request, $response);
    }

    public function findResourceWithCrud(Request\Filter $request, Response $response){
        $request->getFilterCriteria()->addCondition("ORGANIZATION", "=", $this->environment->getCurrentOrganization());
        $request->getFilterCriteria()->addCondition("STATUS", "<>", "D");
        return parent::find($request, $response);
    }

    public function BasicCalendarSave(Request\DataSet $request, Response $response)
    {
        $row = $request->getDataSet()->getRows()[0];

        $id = $row["ID"];
        $calendarDefault = $row["CALENDAR_DEFAULT"];
        $structure = $row["RESOURCE"];
        $calendar = $this->resourceService->saveEditCalendar($id,$calendarDefault,$structure);
        $response->addDataSet(new DataSet('basicCalendarStructure', [$calendar->getId()]));

    }
    public function findBasicCalendar(Request\Filter $request, Response $response)
    {
        $filter = $request->getFilterCriteria()->getConditions();

        $structure = Utilities::getValueInFilterDataSourceByName($filter,'RESOURCE');
        $calendars = $this->resourceService->findBasicCalendar($structure);
        $response->addDataSet(new DataSet('findBasicCalendar',($calendars)));

    }
    public function findCalendarDatesResource(Request\Filter $request, Response $response)
    {
        $filters = $request->getFilterCriteria()->getConditions();
        $month = Utilities::getValueInFilterDataSourceByName($filters,'MONTH');
        $year = Utilities::getValueInFilterDataSourceByName($filters,'YEAR');
        $resourceId = Utilities::getValueInFilterDataSourceByName($filters,'RESOURCE');
        $dataSet = array();
        list($beginDate, $endDate) = Utilities::getBeginAndEndDateByMonth($month,$year);
        if($resourceId){
            $dataSet = $this->resourceService->findExceptionCalendarDatesResource($resourceId,$beginDate, $endDate);
        }
        $response->addDataset(new DataSet('CalendarDetail', $dataSet));
    }
    
    public function findByResourceCalendarDetail(Request\Filter $request, Response $response)
    {
        $filters = $request->getFilterCriteria()->getConditions();
        $id = Utilities::getValueInFilterDataSourceByName($filters,'ID');
        $resourceId = Utilities::getValueInFilterDataSourceByName($filters,'RESOURCE');
        $dataSet = array();
        if($resourceId){
            $dataSet = $this->resourceService->findCalendarExceptionByResource($resourceId,$id);
        }
        DevOpsUtilities::setDataSetDefault($request, $response, $dataSet);
    }
    
    public function findCalendarWeeklyDays(Request\Filter $request, Response $response)
    {
        $filters = $request->getFilterCriteria()->getConditions();
        $id = Utilities::getValueInFilterDataSourceByName($filters,'ID');
        $resourceId = Utilities::getValueInFilterDataSourceByName($filters,'RESOURCE');
        $dataSet = array();
        if($resourceId){
            $dataSet = $this->resourceService->findCalendarWeeklyDays($resourceId,$id);
        }
        DevOpsUtilities::setDataSetDefault($request, $response, $dataSet);
    }

    public function findAllTaskRelatedResource(Request\Filter $request, Response $response)
    {
        $filters = $request->getFilterCriteria()->getConditions();
        $taskId = Utilities::getValueInFilterDataSourceByName($filters,'TASK');

        $dataSet = $this->resourceService->getAllowedResourcesToCompleateTheTask($taskId);
        $response->addDataset(new Dataset('findAllTaskRelatedResource', [$dataSet]));
    }
    public function saveWeeklyDays(Request\DataSet $request, Response $response)
    {
        try{
            $rows = $request->getDataSet()->getRows()[0];

            list($calendar, $id, $typeSunday, $dailyHoursSunday, $typeMonday, $dailyHoursMonday,
                $typeTuesday, $dailyHoursTuesday, $typeWednesday, $dailyHoursWednesday,
                $typeThursday, $dailyHoursThursday, $typeFriday, $dailyHoursFriday,
                $typeSaturday, $dailyHoursSaturday, $beginDate, $endDate, $name) =
                CalendarWeeklyDays::buidingCreateWeekWork($rows);

            $calendarDatail = $this->resourceService->saveWeeklyDays($id, $calendar, $name, $typeSunday, $dailyHoursSunday,$typeMonday,
                $dailyHoursMonday, $typeTuesday, $dailyHoursTuesday, $typeWednesday, $dailyHoursWednesday, $typeThursday,
                $dailyHoursThursday, $typeFriday, $dailyHoursFriday, $typeSaturday, $dailyHoursSaturday, $beginDate, $endDate);
            
            $response->addDataset(new Dataset('calendarWeeklyDaysResource', array('ID' => $calendarDatail->getId())));
            $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_SAVED));
            
        }catch(\Exception $e){
            $response->addMessage(new Response\Message('Erro inesperado :'.$e->getMessage(),Response::STATUS_ERROR));
        }
    }
}
